<?php
/**
 * @package   AkeebaLoginGuard
 * @copyright Copyright (c)2016-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Joomla\Plugin\User\Loginguard\Field;

use Akeeba\Component\LoginGuard\Administrator\Model\BackupcodesModel;
use Akeeba\Component\LoginGuard\Administrator\Model\MethodsModel;
use Akeeba\Component\LoginGuard\Administrator\View\Methods\HtmlView;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Form\FormField;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\User\UserFactoryInterface;

// Prevent direct access
defined('_JEXEC') || die;

class LoginguardField extends FormField
{
	/**
	 * Element name
	 *
	 * @var   string
	 */
	protected $_name = 'Loginguard';

	function getInput()
	{
		// Make sure we can load the classes we need
		if (!class_exists(HtmlView::class, true) || !ComponentHelper::isEnabled('com_loginguard'))
		{
			return Text::_('PLG_USER_LOGINGUARD_ERR_NOCOMPONENT');
		}

		// Load the language files
		$jLang = Factory::getApplication()->getLanguage();
		$jLang->load('com_loginguard', JPATH_ADMINISTRATOR, null, true, true);
		$jLang->load('com_loginguard', JPATH_SITE, null, true, true);

		$user_id = $this->form->getData()->get('id', null);

		if (is_null($user_id))
		{
			return Text::_('PLG_USER_LOGINGUARD_ERR_NOUSER');
		}

		$user = Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById($user_id);

		// Get a model
		/** @var MVCFactoryInterface $factory */
		$factory = Factory::getApplication()->bootComponent('com_loginguard')->getMVCFactory();

		/** @var MethodsModel $methodsModel */
		$methodsModel = $factory->createModel('Methods', 'Administrator');
		/** @var BackupcodesModel $methodsModel */
		$backupCodesModel = $factory->createModel('Backupcodes', 'Administrator');

		// Get a view object
		/** @var HtmlView $view */
		$view = $factory->createView('Methods', 'Site', 'Html', [
			'base_path' => JPATH_SITE . '/components/com_loginguard',
		]);
		$view->setModel($methodsModel, true);
		/** @noinspection PhpParamsInspection */
		$view->setModel($backupCodesModel);
		$view->document  = Factory::getApplication()->getDocument();
		$view->returnURL = base64_encode(Uri::getInstance()->toString());
		$view->user      = $user;

		$view->document->getWebAssetManager()->getRegistry()->addExtensionRegistryFile('com_loginguard');

		@ob_start();
		$view->display();
		return @ob_get_clean();
	}
}
